# AddAppLauncherEntry.ksh

# Script to add entries to the App Launcher table

Usage()
{
  if [ $bNLSMessages = 0 ]; then
    print "USAGE:"
    print ""
    print "  AddAppLauncherEntry.ksh [-S server] {{-U user -P password}|-E} {-f input file | -n name -l platform -c protocol [-m min port -x max port -w wait time -r retry count] -g program -a data}"
    print ""
    print "  Database connection options:"
    print "    -S<server>       Database server"
    print "    -U<user>         User name for the database login"
    print "    -P<password>     Password for the database login"
    print "    -E               Trusted connection"
    print ""
    print "    -f<input file>   Specifies an input file containing new entries."
	print ""
    print "  New entry parameters:"
    print "    -n<name>         RDM Name"
    print "    -l<platform>     Platform"
    print "    -c<protocol>     Protocol"
    print "    -m<min port>     Min Port Number"
    print "    -x<max port>     Max Port Number"
    print "    -w<wait time>    Wait Time"
    print "    -r<retry count>  Retry Count"
    print "    -g<program>      Program Name"
    print "    -a<data>         Program Data"
    print ""
    print "Creates the Application Launcher table: RDM_LAUNCHER"
    print " if it does not exist"
    print "Adds an entry to the Application Launcher table."
    print ""
    print "Prerequisites:"
    print "  MKS Toolkit version 6.1 or higher"
    print "  Microsoft SQL Server 7.0 client"
  else
    ./ASIShellMsg JavaLauncherScriptMsg 1
    ./ASIShellMsg JavaLauncherScriptMsg 2
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 3
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 68
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 4
    ./ASIShellMsg JavaLauncherScriptMsg 5
    ./ASIShellMsg JavaLauncherScriptMsg 6
    ./ASIShellMsg JavaLauncherScriptMsg 7
    ./ASIShellMsg JavaLauncherScriptMsg 8
    ./ASIShellMsg JavaLauncherScriptMsg 9
    ./ASIShellMsg JavaLauncherScriptMsg 10
    ./ASIShellMsg JavaLauncherScriptMsg 11
    ./ASIShellMsg JavaLauncherScriptMsg 12
    ./ASIShellMsg JavaLauncherScriptMsg 13
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 15
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 14
  fi
}

function check_for_required_parameters
{
  # check for the parameter to indicate that the input file should be ignored
  if [ -z "$1" -o "$1" != "1" ]; then
    if [ ! -z "$sFile" -a -f "$sFile" ]; then
      return
    elif [ ! -z "$sFile" -a ! -f "$sFile" ]; then
      if [ $bNLSMessages -eq 1 ]; then
        ./ASIShellMsg JavaLauncherScriptMsg 69 >&2
      else
        echo "ERROR: Input file not found" >&2
      fi
      Usage >&2
      exit 2
    fi
  fi

  if [ -z "$sName" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 16 >&2
	else
      echo "ERROR: Missing required parameter -n<name>" >&2
	fi
    Usage >&2
    exit 2
  fi
  if [ -z "$sPlatform" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 17 >&2
	else
      echo "ERROR: Missing required parameter -l<platform>" >&2
	fi
    Usage >&2
    exit 2
  fi
  if [ -z "$sProtocol" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 18 >&2
	else
      echo "ERROR: Missing required parameter -c<protocol>" >&2
	fi
    Usage >&2
    exit 2
  fi
  if [ -z "$sProgramName" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 23 >&2
	else
      echo "ERROR: Missing required parameter -g<program>" >&2
	fi
    Usage >&2
    exit 2
  fi
  if [ -z "$sProgramData" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 24 >&2
	else
      echo "ERROR: Missing required parameter -a<data>" >&2
	fi
    Usage >&2
    exit 2
  fi
}

# intialize variables
bTrustedConnection=0
sDatabaseName="Object"
sSQLError="Error adding row to table."
sSQLSuccess="Successfully added row to table."
bNLSMessages=0

# default values for non-required params
nMinPort=0
nMaxPort=0
nWaitTime=0
nRetryCount=0

ostype=$(uname -s)
if [ "$ostype" = "Windows_NT" -a -f "ASIShellMsg.exe" ]; then
  bNLSMessages=1
fi

if [ $bNLSMessages -eq 1 ]; then
  sSQLError=`./ASIShellMsg JavaLauncherScriptMsg 30`
  sSQLSuccess=`./ASIShellMsg JavaLauncherScriptMsg 31`
fi

# get command-line options
while getopts U:S:P:D:f:n:l:c:m:x:w:r:g:a:E c; do
  case $c in
  f)  sFile="$OPTARG";;
  n)  sName="$OPTARG";;
  l)  sPlatform="$OPTARG";;
  c)  sProtocol="$OPTARG";;
  m)  nMinPort=$OPTARG;;
  x)  nMaxPort=$OPTARG;;
  w)  nWaitTime=$OPTARG;;
  r)  nRetryCount=$OPTARG;;
  g)  sProgramName="$OPTARG";;
  a)  sProgramData="$OPTARG";;

  U)  user_opt="-U$OPTARG";;
  P)  password_opt="-P$OPTARG";;
  S)  server_opt="-S$OPTARG";;
  E)  bTrustedConnection=1;;
  D)  sDatabaseName="$OPTARG";;

  *)  Usage; exit;;
  esac
done

check_for_required_parameters

# form the SQL Server login string
sql_opt=$server_opt
if [ $bTrustedConnection -eq 1 ]; then
  sql_opt="$sql_opt -E"
else
  sql_opt="$sql_opt $user_opt $password_opt"
fi

function ProcessArg
{
  arg_opt=$(echo "$1" | awk '{ if (substr($0, 1, 1) == "-") printf("%s", substr($0, 2, 1)); else printf("%s", substr($0, 1, 1)); }')
  arg_value=$(echo "$1" | awk '{ if (substr($0, 1, 1) == "-") printf("%s", substr($0, 3, length($0)-2)); else printf("%s", substr($0, 2, length($0)-1)); }')

  case $arg_opt in
  n)  sName="$arg_value";;
  l)  sPlatform="$arg_value";;
  c)  sProtocol="$arg_value";;
  m)  nMinPort=$arg_value;;
  x)  nMaxPort=$arg_value;;
  w)  nWaitTime=$arg_value;;
  r)  nRetryCount=$arg_value;;
  g)  sProgramName="$arg_value";;
  a)  sProgramData="$arg_value";;
  *)  Usage; exit;;
  esac

}

function ProcessRow
{
  osql -n $sql_opt -d$sDatabaseName <<- ENDSQL

	SET NOCOUNT ON
	GO

	-- create the table if it does not exist
	IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RDM_LAUNCHER' AND type = 'U')
	BEGIN
	  CREATE TABLE RDM_LAUNCHER
	  (
	    RDM_NAME		NVARCHAR(32),
		PLATFORM		NVARCHAR(32) PRIMARY KEY (RDM_NAME, PLATFORM),
		PROTOCOL		NVARCHAR(32),
		MIN_PORT		INTEGER DEFAULT 0,
		MAX_PORT		INTEGER DEFAULT 0,
		WAIT_TIME		INTEGER DEFAULT 0,
		RETRY_COUNT		INTEGER DEFAULT 0,
		PROGRAM_NAME	NVARCHAR(1024),
		PROGRAM_DATA	NVARCHAR(1024)
	  )
	END
	GO

	-- add the specified entry to the table
	INSERT INTO RDM_LAUNCHER
	SELECT "$sName", "$sPlatform", "$sProtocol", $nMinPort, $nMaxPort, $nWaitTime, $nRetryCount, "$sProgramName", "$sProgramData"
	IF @@ERROR = 0 AND @@ROWCOUNT = 1
	  PRINT '$sSQLSuccess'
	ELSE
	  PRINT '$sSQLError'
	GO

	SET NOCOUNT OFF
	GO

ENDSQL
}

function ProcessFile
{
  # States:
  #  0:	Looking for a BEGIN_ROW
  #  1:	Just found a BEGIN_ROW
  #  2:	Reading data
  #  3:	Just found END_ROW

  # Read the input file looking for BEGIN_ROW and END_ROW to delimit an entry
  nState=0

  while read f; do

    # if nState=1, then the last line was a BEGIN_ROW, meaning the current line is real data
    if [ $nState -eq 1 ]; then
      nState=2
    fi
    if [ $nState -eq 2 ]; then
      if [ "$f" = "END_ROW" ]; then
        nState=3
      fi
    fi
    # look for the "BEGIN_ROW" indicator if nState=0
    if [ $nState -eq 0 ]; then
      if [ "$f" = "BEGIN_ROW" ]; then
        nState=1
        sName=""
        sPlatform=""
        sProtocol=""
        nMinPort=0
        nMaxPort=0
        nWaitTime=0
        nRetryCount=0
        sProgramName=""
        sProgramData=""
	  fi

    # if nState=2, process as arg
    elif [ $nState -eq 2 ]; then
      ProcessArg "$f"

    # if nState=3, process line
    elif [ $nState -eq 3 ]; then
      check_for_required_parameters "1"
      ProcessRow
      nState=0
    fi
  done < "$sFile"

}

if [ ! -z "$sFile" ]; then
  ProcessFile
else
  ProcessRow
fi