# AddAppLauncherMenuItem.sh

# Script to add a menu item entry for the application launcher

Usage()
{
  if [ $bNLSMessages = 0 ]; then
    print "USAGE:"
    print ""
    print "  AddAppLauncherMenuItem.ksh [-S server] {{-U user -P password}|-E} {-f input file | -n name -c cid -l label -r control id}"
    print ""
    print "  Database connection options:"
    print "    -S<server>       Database server"
    print "    -U<user>         User name for the database login"
    print "    -P<password>     Password for the database login"
    print "    -E               Trusted connection"
    print ""
    print "    -f<input file>   Specifies an input file containing new entries."
    print ""
    print "  App Launcher parameters:"
    print "    -n<name>         RDM Name"
    print ""
    print "  Menu Item parameters:"
    print "    -c<cid>          Class ID of the object"
    print "    -l<label>        label for the menu item"
    print "    -r<control id>   Control ID for the menu item"
    print ""
    print "Creates a menu item for the corresponding Application Launcher entry."
    print ""
    print "Prerequisites:"
    print "  MKS Toolkit version 6.1 or higher"
    print "  Microsoft SQL Server 7.0 client"
  else
    ./ASIShellMsg JavaLauncherScriptMsg 1
    ./ASIShellMsg JavaLauncherScriptMsg 35
    ./ASIShellMsg JavaLauncherScriptMsg 3
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 68
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 4
    ./ASIShellMsg JavaLauncherScriptMsg 5
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 36
    ./ASIShellMsg JavaLauncherScriptMsg 37
    ./ASIShellMsg JavaLauncherScriptMsg 39
    ./ASIShellMsg JavaLauncherScriptMsg 40
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 41
	print ""
    ./ASIShellMsg JavaLauncherScriptMsg 14
  fi
}

function check_for_required_parameters
{
  # check for the parameter to indicate that the input file should be ignored
  if [ -z "$1" -o "$1" != "1" ]; then
    if [ ! -z "$sFile" -a -f "$sFile" ]; then
      return
    elif [ ! -z "$sFile" -a ! -f "$sFile" ]; then
      if [ $bNLSMessages -eq 1 ]; then
        ./ASIShellMsg JavaLauncherScriptMsg 69 >&2
      else
        echo "ERROR: Input file not found" >&2
      fi
      Usage >&2
      exit 2
    fi
  fi

  if [ -z "$sName" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 16 >&2
	else
      echo "ERROR: Missing required parameter -n<name>" >&2
	fi
    Usage >&2
    exit 2
  fi
  if [ -z "$sCID" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 42 >&2
	else
      echo "ERROR: Missing required parameter -c<cid>" >&2
	fi
    Usage >&2
    exit 2
  fi
  if [ -z "$sLabel" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 44 >&2
	else
      echo "ERROR: Missing required parameter -l<label>" >&2
	fi
    Usage >&2
    exit 2
  fi
  if [ -z "$sControlID" ]; then
    if [ $bNLSMessages -eq 1 ]; then
      ./ASIShellMsg JavaLauncherScriptMsg 45 >&2
	else
      echo "ERROR: Missing required parameter -r<control id>" >&2
	fi
    Usage >&2
    exit 2
  fi
}

# intialize variables
bTrustedConnection=0
sDatabaseName="Object"
sLaunchPrefix="Launch_"
sParentMenu="Launch"
bNLSMessages=0
sRDMNotFound="There is no entry in RDM_LAUNCHER for "
sEntryExists="There is already an entry in the MenuItem table for class "
sCreatingParent="Creating the parent \"Launch\" menu..."
sParentExists="The \"Launch\" parent menu already exists..."
sLaunchLabel="&Launch"

ostype=$(uname -s)
if [ "$ostype" = "Windows_NT" -a -f "ASIShellMsg.exe" ]; then
  bNLSMessages=1
fi

if [ $bNLSMessages -eq 1 ]; then
  sRDMNotFound=`./ASIShellMsg JavaLauncherScriptMsg 46`
  sEntryExists=`./ASIShellMsg JavaLauncherScriptMsg 47`
  sCreatingParent=`./ASIShellMsg JavaLauncherScriptMsg 48`
  sParentExists=`./ASIShellMsg JavaLauncherScriptMsg 49`
  sLaunchLabel=`./ASIShellMsg JavaLauncherScriptMsg 50`
fi

# get command-line options
while getopts U:S:P:D:f:n:c:l:r:E c; do
  case $c in

  U)  user_opt="-U$OPTARG";;
  P)  password_opt="-P$OPTARG";;
  S)  server_opt="-S$OPTARG";;
  E)  bTrustedConnection=1;;
  D)  sDatabaseName="$OPTARG";;

  n)  sName="$OPTARG";;
  c)  sCID="$OPTARG";;
  l)  sLabel="$OPTARG";;
  r)  sControlID="$OPTARG";;
  f)  sFile="$OPTARG";;

  *)  Usage; exit;;
  esac
done

check_for_required_parameters

# now that the args are read, modify variables
sRDMNotFound="$sRDMNotFound$sName"
sEntryExists="$sEntryExists$sCID"
nID=0

# form the SQL Server login string
sql_opt=$server_opt
if [ $bTrustedConnection -eq 1 ]; then
  sql_opt="$sql_opt -E"
else
  sql_opt="$sql_opt $user_opt $password_opt"
fi


function ProcessArg
{
  arg_opt=$(echo "$1" | awk '{ if (substr($0, 1, 1) == "-") printf("%s", substr($0, 2, 1)); else printf("%s", substr($0, 1, 1)); }')
  arg_value=$(echo "$1" | awk '{ if (substr($0, 1, 1) == "-") printf("%s", substr($0, 3, length($0)-2)); else printf("%s", substr($0, 2, length($0)-1)); }')

  case $arg_opt in
  n)  sName="$arg_value";;
  c)  sCID="$arg_value";;
  l)  sLabel="$arg_value";;
  r)  sControlID="$arg_value";;
  f)  sFile="$arg_value";;
  *)  Usage; exit;;
  esac

}

function ProcessRow
{
  osql -n $sql_opt -d$sDatabaseName <<- ENDSQL

	SET NOCOUNT ON
	GO

	-- make sure the entry exists in the RDM_LAUNCHER table first
	IF NOT EXISTS (SELECT * FROM RDM_LAUNCHER WHERE RDM_NAME = '$sName')
	BEGIN
	  PRINT '$sRDMNotFound'
	END
	ELSE
	BEGIN
	  -- check to see if the menu item exists for this object class or
	  -- one that it derives from
	  -- This part has been commented to avoid multiple menu item for the JSC entry
        -- IF EXISTS (
	    -- SELECT * FROM MenuItem
		-- WHERE name = '$sLaunchPrefix$sName' AND obj_cid IN (
		  -- SELECT DISTINCT base_cid FROM isa_chain WHERE cid = '$sCID'
		-- )
	  -- )
	    -- PRINT '$sEntryExists'
	  -- ELSE
	  -- BEGIN
	    DECLARE @parent_id INT -- the id of the parent menu
		DECLARE @ret INT
	    -- check to see if the parent 'Launch' menu already exists
	    IF NOT EXISTS (
	      SELECT * FROM MenuItem
		  WHERE name = '$sParentMenu' AND obj_cid IN (
		    SELECT DISTINCT base_cid FROM isa_chain WHERE cid = '$sCID'
		  )
	    )
		BEGIN
		  PRINT '$sCreatingParent'
		  DECLARE @prev_id INT
		  EXEC asisp_lookupmenuitem 'MNO2/0/View', @prev_id OUT, @parent_id OUT
		  EXEC @ret = asisp_definemenuitem
		    @name      = '$sParentMenu',
			@obj_cid   = '$sCID',
			@obj_id    = $nID,
			@label     = '$sLaunchLabel',
			@parent_id = @parent_id,
			@prev_id   = @prev_id,
			@flags     = 1342177288,
	        @security  = '',
	        @override  = ''
		END
		ELSE
		  PRINT '$sParentExists'

	    -- get the "Launch" menu item id
		SELECT @parent_id = id
		FROM MenuItem
		WHERE name = '$sParentMenu' AND obj_cid IN (SELECT DISTINCT base_cid FROM isa_chain WHERE cid = '$sCID')

	    -- create the menu item entry
		EXEC @ret = asisp_definemenuitem
		  @name       = '$sLaunchPrefix$sName',
		  @obj_cid    = '$sCID',
		  @obj_id     = $nID,
		  @label      = '$sLabel',
		  @parent_id  = @parent_id,
		  @prev_id    = NULL,
		  @flags      = 1342308352,
		  @control_id = '$sControlID',
	      @security   = '',
	      @override   = ''
	  -- END
	END

	SET NOCOUNT OFF
	GO

ENDSQL
}

function ProcessFile
{
  # States:
  #  0:	Looking for a BEGIN_ROW
  #  1:	Just found a BEGIN_ROW
  #  2:	Reading data
  #  3:	Just found END_ROW

  # Read the input file looking for BEGIN_ROW and END_ROW to delimit an entry
  nState=0

  while read f; do

    # if nState=1, then the last line was a BEGIN_ROW, meaning the current line is real data
    if [ $nState -eq 1 ]; then
      nState=2
    fi
    if [ $nState -eq 2 ]; then
      if [ "$f" = "END_ROW" ]; then
        nState=3
      fi
    fi
    # look for the "BEGIN_ROW" indicator if nState=0
    if [ $nState -eq 0 ]; then
      if [ "$f" = "BEGIN_ROW" ]; then
        nState=1
        sName=""
        sCID=""
        sLabel=""
        sControlID=""
	  fi

    # if nState=2, process as arg
    elif [ $nState -eq 2 ]; then
      ProcessArg "$f"

    # if nState=3, process line
    elif [ $nState -eq 3 ]; then
      check_for_required_parameters "1"
      ProcessRow
      nState=0
    fi
  done < "$sFile"
}

if [ ! -z "$sFile" ]; then
  ProcessFile
else
  ProcessRow
fi